/*
 * LISTING 3: dii_client.cc
 */

#include <iostream.h>
#include <fstream.h>
#include <CORBA.h>
#include "support.h"

/*
 * Ausgabe eines out/inout Parameters
 */
void printReturnParameter(CORBA::NamedValue* value)
{
    CORBA::Flags flags = value->flags();

    if(flags == CORBA::ARG_OUT || flags == CORBA::ARG_INOUT) {
        printParameter(value);
    }
}

/*
 * Hinzufuegen eines Parameters zu einem Request
 */
void addClientParameter(CORBA::Request_ptr req)
{
    char                name[100];
    char                buffer[100];
    CORBA::ArgModeFlags direction;
    int                 loopDone;
    CORBA::Long         count;

    cout << "Parameter Name: ";
    cin  >> name;

    count = req->arguments()->count();

    do {
        loopDone = TRUE;    

        cout << "Richtung: (i)n, (o)ut ";
        cin  >> buffer;

        if(strchr(buffer, 'i')) {
            if(strchr(buffer, 'o')) {
                direction = CORBA::ARG_INOUT;
            } else {
                direction = CORBA::ARG_IN;
            }
        } else if(strchr(buffer, 'o')) {
            direction = CORBA::ARG_OUT;
        } else {
            loopDone = FALSE;
        }
    } while(!loopDone);

    switch(direction) {
        case CORBA::ARG_IN:  
            req->add_in_arg(name);
            queryTypeValue(req->arguments()->item(count)->value());
            break;
        case CORBA::ARG_OUT: 
            req->add_out_arg(name);
            req->arguments()->item(count)->value()->type(queryType());
            break;
        case CORBA::ARG_INOUT:
            req->add_inout_arg(name);
            queryTypeValue(req->arguments()->item(count)->value());
    }
}

/*
 * Hauptroutine
 */
int main(int argc, char* argv[]) 
{

    char        buffer[100];
    char        stringifiedReference[1000];
    char        methodName[100];
    CORBA::Long parameterCount;
    CORBA::Long n;
    int         loopDone = FALSE;

    /*
     * Initialisierung des ORB/BOA
     */
    CORBA::ORB_var orb = CORBA::ORB_init(argc, argv, "mico-local-orb");
    CORBA::BOA_var boa = orb->BOA_init(argc, argv, "mico-local-boa");

    /*
     * Objektreferenz erstellen
     */
    ifstream referenceStream("object.ref");
    referenceStream >> stringifiedReference;
    referenceStream.close();
    CORBA::Object_var obj = orb->string_to_object(stringifiedReference);

    /*
     * Methodenaufruf erstellen
     */
    cout << "Name der aufzurufenden Methode: "; 
    cin  >> methodName;
    cout << "Anzahl der Parameter: ";
    cin  >> parameterCount;

    CORBA::Request_var req = obj->_request(methodName);

    for(n = 0; n < parameterCount; n++) {
        addClientParameter(req);
    }

    cout << "Typ des Rueckgabewertes" << endl;
    req->result()->value()->type(queryType());

    /*
     * Aufruf durchfuehren
     */
    req->invoke();

    /*
     * Auf Ausnahme ueberpruefen
     */
    CORBA::Exception* except = req->env()->exception();
    if(except) {
        cout << "Ausnahme aufgetreten!" << endl;
    } else {
        /*
         * out/inout Parameter ausgeben
         */
        for(n = 0; n < parameterCount; n++) {
            printReturnParameter(req->arguments()->item(n));
        }
     
        /*
         * Rueckgabewert ausgeben 
         */
        cout << "Rueckgabewert: ";
        printAny(req->result()->value());
        cout << endl;
    }

    return 0;    
}