/*
 * LISTING 4: dsi_server.cc
 */

#include <stdlib.h>
#include <iostream.h>
#include <fstream.h>
#include <CORBA.h>
#include "support.h"

/*
 * Ausgabe eines in/inout Parameters
 */
void printGivenParameter(CORBA::NamedValue* value)
{
    CORBA::Flags flags = value->flags();

    if(flags == CORBA::ARG_IN || flags == CORBA::ARG_INOUT) {
        cout << "Parameterwert: ";
        printAny(value->value());
        cout << endl;
    }
}

/*
 * Abfrage eines out/inout Parameters
 */
void queryOutParameter(CORBA::NamedValue* value)
{
    CORBA::Flags flags = value->flags();

    if(flags == CORBA::ARG_OUT || flags == CORBA::ARG_INOUT) {
        queryValue(value->value());
    }
}

/*
 * Hinzufuegen eines Parameters zur Anfrage-Schablone
 */
void addServerParameter(CORBA::NVList_ptr paramList, CORBA::Long itemCnt)
{
    char                buffer[10];
    CORBA::ArgModeFlags direction;
    int                 loopDone;

    do {
        loopDone = TRUE;    

        cout << "Richtung: (i)n, (o)ut ";
        cin  >> buffer;

        if(strchr(buffer, 'i')) {
            if(strchr(buffer, 'o')) {
                direction = CORBA::ARG_INOUT;
            } else {
                direction = CORBA::ARG_IN;
            }
        } else if(strchr(buffer, 'o')) {
            direction = CORBA::ARG_OUT;
        } else {
            loopDone = FALSE;
        }
    } while(!loopDone);

    paramList->add(direction);
    paramList->item(itemCnt)->value()->type(queryType());
}

/*
 * Methodenanfrage interaktiv beantworten
 */
int queryRequest(CORBA::ServerRequest_ptr request)
{
    CORBA::ULong      paramCnt;
    CORBA::ULong      n;
    CORBA::NVList_ptr paramList;
    int               retVal = FALSE;

    cout << "Aufgerufene Methode: " << request->op_name() << endl;
    cout << "Anzahl der Parameter: ";
    cin  >> paramCnt;
    
    paramList = new CORBA::NVList(paramCnt);

    for(n = 0; n < paramCnt; n++) {
        addServerParameter(paramList, n);
    }

    /*
     * Parameterliste an POA zur Pruefung reichen
     */
    if(request->params(paramList)) {
        /*
         * Parameterliste i.O.
         */
        
        /*
         * Uebergebene Parameter ausgeben
         */
        for(n = 0; n < paramCnt; n++) {
            printGivenParameter(paramList->item(n));
        }

        /*
         * Ausgabeparameter abfragen
         */
        for(n = 0; n < paramCnt; n++) {
            queryOutParameter(paramList->item(n));
        }

        CORBA::Any* result = new CORBA::Any();

        queryTypeValue(result);

        request->result(result);

        retVal = true;
    } else {
        /*
         * Parameterlist stimmt nicht ueberein, Ausnahme an Client zurueckliefern
         */
        CORBA::Exception * except = new CORBA::BAD_OPERATION (0, CORBA::COMPLETED_NO);
        request->exception(except);
    }

    return retVal;
}

/*
 * Implementierung eines Servants
 */
class DemoServant : virtual public PortableServer::DynamicImplementation {
public:
    void                invoke(CORBA::ServerRequest_ptr request);
    CORBA::RepositoryId _primary_interface(const PortableServer::ObjectId& oid, PortableServer::POA_ptr poa);
}; 

/*
 * Allgemeiner Methodenaufruf (Dispatcher-Routine)
 */
void DemoServant::invoke(CORBA::ServerRequest_ptr request)
{
    queryRequest(request);
}

/*
 * Zurueckliefern der Schnittstelle
 */
CORBA::RepositoryId DemoServant::_primary_interface(const PortableServer::ObjectId& oid, PortableServer::POA_ptr poa)
{
    return CORBA::string_dup("DSI:NoType:1.0");
}

main(int argc, char *argv[])
{
    /*
     * ORB / POA Initialisierung
     */
    CORBA::ORB_var orb = CORBA::ORB_init (argc, argv, "mico-local-orb");
    CORBA::Object_var poaobj = orb->resolve_initial_references("RootPOA");
    PortableServer::POA_var poa = PortableServer::POA::_narrow (poaobj);
    PortableServer::POAManager_var mgr = poa->the_POAManager();

    /*
     * Servant instatiieren und aktivieren
     */
    DemoServant * servant = new DemoServant();

    PortableServer::ObjectId_var oid = poa->activate_object(servant);

    /*
     * Objektreferenz bekanntmachen
     */
    ofstream referenceStream("object.ref");
    referenceStream << orb->object_to_string(poa->id_to_reference(oid)) << endl;
    referenceStream.close();

    /*
     * Anfragen bedienen
     */
    mgr->activate ();
    orb->run();

    /*
     * Aufraeumen
     */
    poa->destroy (TRUE, TRUE);
    delete servant;
    return 0;
}
