#!/usr/bin/perl -w 

use Getopt::Std;       # Fr Optionen
use HTML::HeadParser;  # HTML-Parser
use HTML::Entities;    # <, > etc. kodieren
use File::Find;
use strict;

# Auswertung der Kommandozeile 
getopts("t:")   || die "Wrong parameter.\n"; ; 

if(defined $main::opt_t) {
  print "checking only files not older than  ",
    $main::opt_t, " days !\n"; 
}

if   ($#ARGV != 0) { 
  print STDERR "Wrong number of parameters.\n";   
} else {
  ($main::startdir) =  @ARGV;   
}

# Liste der Refenzen auf die Dateien anlegen
my(@lrFiles) ;

# find fr Analyse der HTML-Dateien
File::Find::find(\&fc, $main::startdir );

# Header der generierten Datei schreiben
print <<PAGEBEGIN;
<HTML>
<HEAD>
   <TITLE> Seitenverzeichnis </TITLE>
</HEAD>
PAGEBEGIN

# Liste der Listreferencen bearbeiten
my($listref,$element,$OldDeep,$i);
$OldDeep = ($main::startdir =~ tr@/@@);

# Liste der URLs alphabetisch sortieren 
my(@liste) = sort my_sort @main::lrFiles;

# Erzeugen der eigentlichen Liste der Urls
# Je tiefer, desto mehr wird eingerckt

foreach $listref ( @liste) {

  my($Deep,$url,$titel) = @$listref;
  if ($Deep > $OldDeep)  {  
    for ($i =1;$i <= ($Deep - $OldDeep);$i++){
      print  "   " x  $Deep . "<ul> \n";
    }
  }
  if ($Deep < $OldDeep)  {
    for ($i =1;$i <= ($OldDeep - $Deep);$i++){
      print  "   " x  $Deep . "</ul> \n";
    }
  }
  # Markiere Dokumente ohne Titel
  $titel = "Kein Titel" 
     unless (defined $titel && length($titel));
  # Sonderzeichen im Titel schtzen 
  # und Eintrag ausgeben
  encode_entities($titel);
  print "   " x  $Deep .  
    "  <li> <a href=\"$url\"> $titel </a> \n";
  $OldDeep = $Deep;
   
}

# Footers der generierten Datei schreiben
print "</UL>\n" ;
print <<PAGEEND;
</BODY>
</HTML>
PAGEEND


# HTML-Dateien finden, Tiefe im Verzeichnis 
# und Titel feststellen 
sub fc {
    
  my($Path)     = $main::startdir;
  my($FileName) = $_;    
    
  return unless -f; # no directories
  # Nur Dateien vom Typ *.htm und *.html

  return unless ($FileName =~ /.*\.html?$/ ) ; 

  if(defined $main::opt_t) {
   return if ($main::opt_t <  (-M $FileName) );  
  }

  # URL zusammenbasteln
  my $Url =  $File::Find::name;
  # der Aufrufpfad wird ersetzt 
  $Url =~ s@$Path/@@;
  
  #Tiefe des Baumes feststellen
  my $tiefe = $Url =~ tr@/@@; 

  # Header analysieren
  my $p = HTML::HeadParser->new();
  my $html =  $p->parse_file($FileName);
   # Titel  und  Ergebnis  auf  Liste 
   # von Listenrefenzen schieben
  push(@main::lrFiles, [$tiefe,$Url,$html->header('Title')] );
  }


# Funktion zum sortieren der url
  sub my_sort {
  my $url1 = @$a[1];
  my $url2 = @$b[1];

  my ($wo,$nr);
  $url1 =~ /(?=\/index\.html?)/g;
  $url2 =~ /(?=\/index\.html?)/g;
  my $wo1 = pos $url1;
  my $wo2 = pos $url2;
  $wo1 = 0  unless (defined $wo1);
  $wo2 = 0  unless (defined $wo2);

  # Normal sortieren, wenn keine Index-Datei dabei
  unless ($wo1 || $wo2) {
    return ($url1 cmp $url2);
  }

  $wo = $wo1 > $wo2 ? $wo1 : $wo2;
  
  # Falls beide die  gleiche Anfangswurzel haben 
  #       wird das index File als kleiner zurueckgegeben 
  if (substr($url1,0,$wo) eq substr($url2,0,$wo)) {
    return  $wo2 - $wo1 ; 
  }
  return ($url1 cmp $url2);
}

